@@ -12,11 +12,12 @@ from operator import attrgetter
 from unicodedata import normalize
 
 import numpy as np
-
+from astropy.time import Time as astrotime
 from mosfit import __author__, __contributors__, __version__
 from mosfit.fitter import Fitter
 from mosfit.printer import Printer
 from mosfit.utils import get_mosfit_hash, is_master, open_atomic, speak
+from six import string_types
 
 
 class SortingHelpFormatter(argparse.ArgumentDefaultsHelpFormatter):
@@ -78,7 +79,6 @@ def get_parser(only=None, printer=None):
         '--walker-paths',
         '-w',
         dest='walker_paths',
-        default=[],
         nargs='+',
         help=prt.text('parser_walker_paths'))
 
@@ -97,6 +97,49 @@ def get_parser(only=None, printer=None):
         nargs='+',
         help=prt.text('parser_limiting_magnitude'))
 
+    parser.add_argument(
+        '--prefer-fluxes',
+        dest='prefer_fluxes',
+        default=False,
+        action='store_true',
+        help=prt.text('parser_prefer_fluxes'))
+
+    parser.add_argument(
+        '--time-list',
+        '--extra-times',
+        dest='time_list',
+        default=[],
+        nargs='+',
+        help=prt.text('parser_time_list'))
+
+    parser.add_argument(
+        '--extra-dates',
+        dest='date_list',
+        default=[],
+        nargs='+',
+        help=prt.text('parser_time_list'))
+
+    parser.add_argument(
+        '--extra-mjds',
+        dest='mjd_list',
+        default=[],
+        nargs='+',
+        help=prt.text('parser_time_list'))
+
+    parser.add_argument(
+        '--extra-jds',
+        dest='jd_list',
+        default=[],
+        nargs='+',
+        help=prt.text('parser_time_list'))
+
+    parser.add_argument(
+        '--extra-phases',
+        dest='phase_list',
+        default=[],
+        nargs='+',
+        help=prt.text('parser_time_list'))
+
     parser.add_argument(
         '--band-list',
         '--extra-bands',
@@ -189,6 +232,14 @@ def get_parser(only=None, printer=None):
         nargs='?',
         help=prt.text('parser_iterations'))
 
+    parser.add_argument(
+        '--generative',
+        '-G',
+        dest='generative',
+        default=False,
+        action='store_true',
+        help=prt.text('parser_generative'))
+
     parser.add_argument(
         '--smooth-times',
         '--plot-points',
@@ -196,7 +247,7 @@ def get_parser(only=None, printer=None):
         dest='smooth_times',
         type=int,
         const=0,
-        default=20,
+        default=21,
         nargs='?',
         action='store',
         help=prt.text('parser_smooth_times'))
@@ -219,6 +270,13 @@ def get_parser(only=None, printer=None):
         nargs=2,
         help=prt.text('parser_limit_fitting_mjds'))
 
+    parser.add_argument(
+        '--output-path',
+        '-o',
+        dest='output_path',
+        default='',
+        help=prt.text('parser_output_path'))
+
     parser.add_argument(
         '--suffix',
         '-s',
@@ -239,7 +297,6 @@ def get_parser(only=None, printer=None):
         '-T',
         dest='num_temps',
         type=int,
-        default=1,
         help=prt.text('parser_num_temps'))
 
     parser.add_argument(
@@ -303,7 +360,6 @@ def get_parser(only=None, printer=None):
         '-f',
         dest='frack_step',
         type=int,
-        default=50,
         help=prt.text('parser_frack_step'))
 
     parser.add_argument(
@@ -333,8 +389,8 @@ def get_parser(only=None, printer=None):
         '-R',
         dest='run_until_converged',
         type=float,
-        default=None,
-        const=1.1,
+        default=False,
+        const=True,
         nargs='?',
         help=prt.text('parser_run_until_converged'))
 
@@ -361,15 +417,19 @@ def get_parser(only=None, printer=None):
         '-M',
         dest='maximum_memory',
         type=float,
-        default=np.inf,
         help=prt.text('parser_maximum_memory'))
 
+    parser.add_argument(
+        '--seed',
+        dest='seed',
+        type=int,
+        help=prt.text('parser_seed'))
+
     parser.add_argument(
         '--draw-above-likelihood',
         '-d',
         dest='draw_above_likelihood',
         type=float,
-        default=False,
         const=True,
         nargs='?',
         help=prt.text('parser_draw_above_likelihood'))
@@ -378,16 +438,16 @@ def get_parser(only=None, printer=None):
         '--gibbs',
         '-g',
         dest='gibbs',
-        default=False,
-        action='store_true',
+        action='store_const',
+        const=True,
         help=prt.text('parser_gibbs'))
 
     parser.add_argument(
         '--save-full-chain',
         '-c',
         dest='save_full_chain',
-        default=False,
-        action='store_true',
+        action='store_const',
+        const=True,
         help=prt.text('parser_save_full_chain'))
 
     parser.add_argument(
@@ -486,6 +546,16 @@ def get_parser(only=None, printer=None):
         action='store_true',
         help=prt.text('parser_local_data_only'))
 
+    parser.add_argument(
+        '--method',
+        '-D',
+        dest='method',
+        type=str,
+        const='select',
+        default='ensembler',
+        nargs='?',
+        help=prt.text('parser_method'))
+
     return parser
 
 
@@ -556,14 +626,83 @@ def main():
         prt.message('enabling_s')
         args.smooth_times = 0
 
-    changed_iterations = False
+    args.method = 'nester' if args.method.lower() in [
+        'nest', 'nested', 'nested_sampler', 'nester'] else 'ensembler'
+
+    if args.method == 'nester':
+        if args.run_until_converged and args.iterations >= 0:
+            raise ValueError(prt.text('R_i_mutually_exclusive'))
+        if args.walker_paths is not None:
+            raise ValueError(prt.text('w_nester_mutually_exclusive'))
+
+    if args.generative:
+        if args.iterations > 0:
+            prt.message('generative_supercedes', warning=True)
+        args.iterations = 0
+
+    no_events = False
     if args.iterations == -1:
         if len(args.events) == 0:
-            changed_iterations = True
+            no_events = True
             args.iterations = 0
         else:
             args.iterations = 5000
 
+    if len(args.date_list):
+        if no_events:
+            prt.message('no_dates_gen', warning=True)
+        else:
+            args.time_list = [str(astrotime(x.replace('/', '-')).mjd)
+                              for x in args.date_list]
+            args.time_unit = 'mjd'
+
+    if len(args.mjd_list):
+        if no_events:
+            prt.message('no_dates_gen', warning=True)
+        else:
+            args.time_list = args.mjd_list
+            args.time_unit = 'mjd'
+
+    if len(args.jd_list):
+        if no_events:
+            prt.message('no_dates_gen', warning=True)
+        else:
+            args.time_list = [str(astrotime(
+                float(x), format='jd').mjd) for x in args.jd_list]
+            args.time_unit = 'mjd'
+
+    if len(args.phase_list):
+        if no_events:
+            prt.message('no_dates_gen', warning=True)
+        else:
+            args.time_list = args.phase_list
+            args.time_unit = 'phase'
+
+    if len(args.time_list):
+        if any([any([y in x]) for y in ['-', '/'] for x in args.time_list]):
+            try:
+                args.time_list = [astrotime(
+                    x.replace('/', '-')).mjd for x in args.time_list]
+            except ValueError:
+                if len(args.time_list) == 1 and isinstance(
+                        args.time_list[0], string_types):
+                    args.time_list = args.time_list[0].split()
+                args.time_list = [float(x) for x in args.time_list]
+                args.time_unit = 'phase'
+        else:
+            if any(['+' in x for x in args.time_list]):
+                args.time_unit = 'phase'
+            args.time_list = [float(x) for x in args.time_list]
+
+        if min(args.time_list) > 2400000:
+            prt.message('assuming_jd')
+            args.time_list = [x - 2400000.5 for x in args.time_list]
+            args.time_unit = 'mjd'
+        elif min(args.time_list) > 50000:
+            prt.message('assuming_mjd')
+            args.time_unit = 'mjd'
+        args.time_unit = None
+
     if args.burn is None and args.post_burn is None:
         args.burn = int(np.floor(args.iterations / 2))
 
@@ -571,16 +710,26 @@ def main():
         args.fracking = False
 
     if (args.run_until_uncorrelated is not None and
-            args.run_until_converged is not None):
+            args.run_until_converged):
         raise ValueError(
             '`-R` and `-U` options are incompatible, please use one or the '
             'other.')
-    elif args.run_until_uncorrelated is not None:
+    if args.run_until_uncorrelated is not None:
         args.convergence_type = 'acor'
         args.convergence_criteria = args.run_until_uncorrelated
-    elif args.run_until_converged is not None:
-        args.convergence_type = 'psrf'
-        args.convergence_criteria = args.run_until_converged
+    elif args.run_until_converged:
+        if args.method == 'ensembler':
+            args.convergence_type = 'psrf'
+            args.convergence_criteria = (
+                1.1 if args.run_until_converged is True else
+                args.run_until_converged)
+        else:
+            args.convergence_type = 'dlogz'
+
+    if args.method == 'nester':
+        args.convergence_criteria = (
+            0.02 if args.run_until_converged is True else
+            args.run_until_converged)
 
     if is_master():
         # Get hash of ourselves
@@ -611,6 +760,23 @@ def main():
 
         upload_token_path = os.path.join(dir_path, 'cache', 'dropbox.token')
 
+        if args.method == 'nester':
+            unused_args = [
+                [args.burn, '-b'],
+                [args.post_burn, '-p'],
+                [args.frack_step, '-f'],
+                [args.num_temps, '-T'],
+                [args.run_until_uncorrelated, '-U'],
+                [args.draw_above_likelihood, '-d'],
+                [args.gibbs, '-g'],
+                [args.save_full_chain, '-c'],
+                [args.maximum_memory, '-M']
+            ]
+            for ua in unused_args:
+                if ua[0] is not None:
+                    prt.message('argument_not_used',
+                                reps=[ua[1], '-D nester'], warning=True)
+
         # Perform a few checks on upload before running (to keep size
         # manageable)
         if args.upload and not args.test and args.smooth_times > 100:
@@ -674,7 +840,7 @@ def main():
 
         args.upload_token = upload_token
 
-        if changed_iterations:
+        if no_events:
             prt.message('iterations_0', wrapped=True)
 
         # Create the user directory structure, if it doesn't already exist.
@@ -749,6 +915,26 @@ def main():
                         os.path.join(dir_path, 'models', mdir, mfil),
                         os.path.join(fil_path))
 
+    # Set some default values that we checked above.
+    if args.frack_step == 0:
+        args.fracking = False
+    elif args.frack_step is None:
+        args.frack_step = 50
+    if args.burn is None and args.post_burn is None:
+        args.burn = int(np.floor(args.iterations / 2))
+    if args.draw_above_likelihood is None:
+        args.draw_above_likelihood = False
+    if args.maximum_memory is None:
+        args.maximum_memory = np.inf
+    if args.gibbs is None:
+        args.gibbs = False
+    if args.save_full_chain is None:
+        args.save_full_chain = False
+    if args.num_temps is None:
+        args.num_temps = 1
+    if args.walker_paths is None:
+        args.walker_paths = []
+
     # Then, fit the listed events with the listed models.
     fitargs = vars(args)
     Fitter(**fitargs).fit_events(**fitargs)
