@@ -16,6 +16,15 @@ class Likelihood(Module):
 
     MIN_COV_TERM = 1.0e-30
 
+    def __init__(self, **kwargs):
+        """Initialize `Likelihood` module."""
+        super(Likelihood, self).__init__(**kwargs)
+        self._cuda_reported = False
+        self._use_cpu = None
+
+        if not self._model._fitter._cuda:
+            self._use_cpu = True
+
     def process(self, **kwargs):
         """Calculate the likelihood, returning ln(likelihood)."""
         ret = {'value': LIKELIHOOD_FLOOR}
@@ -63,12 +72,20 @@ class Likelihood(Module):
             if condn > 1.0e10:
                 return ret
 
-            use_cpu = True
-            if self._model._fitter._cuda:
-                use_cpu = False
+            if self._use_cpu is not True and self._model._fitter._cuda:
                 try:
                     import pycuda.gpuarray as gpuarray
                     import skcuda.linalg as skla
+                except ImportError:
+                    self._use_cpu = True
+                    if not self._cuda_reported:
+                        self._printer.message(
+                            'cuda_not_enabled', master_only=True, warning=True)
+                else:
+                    self._use_cpu = False
+                    if not self._cuda_reported:
+                        self._printer.message('cuda_enabled', master_only=True)
+                        self._cuda_reported = True
 
                     kmat_gpu = gpuarray.to_gpu(kmat)
                     # kmat will now contain the cholesky decomp.
@@ -81,10 +98,8 @@ class Likelihood(Module):
                     value -= (0.5 * (
                         skla.mdot(skla.transpose(res_gpu),
                                   cho_mat_gpu)).get())[0][0]
-                except ImportError:
-                    use_cpu = True
 
-            if use_cpu:
+            if self._use_cpu:
                 try:
                     chol_kmat = scipy.linalg.cholesky(kmat, check_finite=False)
 
@@ -110,6 +125,10 @@ class Likelihood(Module):
         else:
             # Shortcut when matrix is diagonal.
             self._o_band_vs = kwargs['obandvs']
+            # print('likelihood')
+            # print(np.sqrt(diag))
+            # print(self._o_band_vs)
+            # print(residuals)
             value = -0.5 * np.sum(
                 residuals ** 2 / (self._o_band_vs ** 2 + diag) +
                 np.log(self._o_band_vs ** 2 + diag))
