@@ -5,7 +5,6 @@ import numexpr as ne
 import numpy as np
 from astropy import constants as c
 from astropy import units as u
-
 from mosfit.constants import FOUR_PI
 from mosfit.modules.seds.sed import SED
 
@@ -36,36 +35,42 @@ class Blackbody(SED):
         fc = self.FLUX_CONST  # noqa: F841
         cc = self.C_CONST
         temperature_phot = self._temperature_phot
+
+        # Some temp vars for speed.
         zp1 = 1.0 + kwargs[self.key('redshift')]
+        Azp1 = u.Angstrom.cgs.scale / zp1
+        czp1 = cc / zp1
+
         seds = []
+        rest_wavs_dict = {}
         evaled = False
+
         for li, lum in enumerate(self._luminosities):
-            radius_phot = self._radius_phot[li]  # noqa: F841
-            temperature_phot = self._temperature_phot[li]  # noqa: F841
             bi = self._band_indices[li]
             if lum == 0.0:
-                if bi >= 0:
-                    seds.append(np.zeros_like(self._sample_wavelengths[bi]))
-                else:
-                    seds.append([0.0])
+                seds.append(np.zeros(len(
+                    self._sample_wavelengths[bi]) if bi >= 0 else 1))
                 continue
+
             if bi >= 0:
-                rest_wavs = (self._sample_wavelengths[bi] *
-                             u.Angstrom.cgs.scale / zp1)
+                rest_wavs = rest_wavs_dict.setdefault(
+                    bi, self._sample_wavelengths[bi] * Azp1)
             else:
-                rest_wavs = [cc / (self._frequencies[li] * zp1)]  # noqa: F841
+                rest_wavs = np.array(  # noqa: F841
+                    [czp1 / self._frequencies[li]])
+
+            radius_phot = self._radius_phot[li]  # noqa: F841
+            temperature_phot = self._temperature_phot[li]  # noqa: F841
 
             if not evaled:
-                sed = ne.evaluate(
+                seds.append(ne.evaluate(
                     'fc * radius_phot**2 / rest_wavs**5 / '
-                    'expm1(xc / rest_wavs / temperature_phot)')
+                    'expm1(xc / rest_wavs / temperature_phot)'))
                 evaled = True
             else:
-                sed = ne.re_evaluate()
+                seds.append(ne.re_evaluate())
 
-            sed[np.isnan(sed)] = 0.0
-
-            seds.append(sed)
+            seds[-1][np.isnan(seds[-1])] = 0.0
 
         seds = self.add_to_existing_seds(seds, **kwargs)
 
